#include<math.h>
#include<stdio.h>
#include<stdlib.h>

#define error 1.e-10
#define a 1.4
#define b 0.3
#define N 32
#define pi acos(-1.0)


int componentsManifold(double *, double, double);
double functionalEqErr(double *,double, double);
double fundamDomain(double *, double, int);
void Manifold(double *C, double s0, double alpha, double delta, int indicator,int);
void Fn(double z[2] ,double zn[2], int );
void evaluate(double zn[2],double s, double *C, double alpha, double delta);
double hornerEvaluation(double *C,double s0);

int main(void){
    /*main idea: In this program we try to sketch the Henon attractor, Estable Manifold and Inestable Manifold of fixed point(we have two fixed point depen on the a and b could be all saddle-node or one atractor and another slow atractor) 
    componentsManifold: give the coefficient of the expansion of the estable manifold or inestable manifold.
    functionalEqErr: return the error of functional equation |F(Wn(s)-wn(lambda s)| 
    fundamDomain: return the fundamental domain [0,deltaf](indicator 1) or [-deltab,0](indicator = -1) 
    nextpoint: give s0 nextpoint return s1 such that dmin<|w(s1)-w(s0)| < dmax
    Fn: return the n-th evaluation of F at point x
    evaluate: return the Fn(w(s/(lambda)^n))
    */
    int i,choice, count;
    double alpha1, alpha2, fixPoint1x, fixPoint2x,fixedPointX;
    double dmin, dmax,delta1f,delta1b,delta2f, delta2b, delta1, delta2;
    double point, point1f, point2f,point1b, point2b, s,apprX, apprY;
    double *eta1, *eta2, z1[2],z0[2];
    double dif;
    
    /*alpha1 alpha2 are eigenvalues of fixed point that |alpha1|>1 and |alpha2|<1*/
    /*gamma1, gamma2 are the second component of the expansion of unestable and estable manifold respectly */
    
    dmin = 1.e-5;
    dmax = 1.e-1;
    point1f = 0;
    point1b = 0;
    point2f = 0;
    point2b = 0;
    count = 0;
    delta1 = 0;
    delta2 = 0;
    
    eta1 = (double *)malloc(N*sizeof(double));
    eta2 = (double *)malloc(N*sizeof(double));
    
    if (eta1 == NULL || eta2 == NULL){
        printf("problem of memory \n");
        exit;
    }
    
    FILE *file1;
    FILE *file2;
    FILE *file3;
  
    file2 = fopen("ExpansionManiEstableY","w");
    file1 = fopen("ExpansionManiInestableY", "w");
    file3 = fopen("HenonAttractor","w");
   
    if(file1 == NULL || file2 == NULL || file3 == NULL){
        printf("problem in opening output files\n");
    }
    
    /*fixPoint1x is the first component of fixedPoint and x = fixPoint1x y=0.3*x*/
    fixPoint1x = (b-1 + sqrt((b-1)*(b-1)+4*a))/(2*a);
    fixPoint2x = (b-1 - sqrt((b-1)*(b-1)+4*a))/(2*a);
    
    printf("we have two fixed points, which one do you prefer?\n");
    printf("the first fixed point %le %le\n", fixPoint1x,0.3*fixPoint1x);
    printf("the second fixed point %le %le\n", fixPoint2x,0.3*fixPoint2x);
    printf("enter 1 for fixed point 1 and enter 2 for fixed point 2\n");
    scanf("%d",&choice);
    
    if(choice == 1){
        fixedPointX = fixPoint1x;
    }else{
        fixedPointX = fixPoint2x;
    }
    
    printf("to sketch the  Henon Attractor we need a point near Fixed Point, so enter the difference for each component (difX, difY)\n");
    printf("For instance, 0.001 for x \n 0.001 for y\n");
    scanf("%le", &apprX);
    scanf("%le", &apprY);
    
    
    alpha2 = -a*fixedPointX + sqrt((2*a*fixedPointX)*(2*a*fixedPointX) + 4*b)/2;
    alpha1 = -a*fixedPointX - sqrt((2*a*fixedPointX)*(2*a*fixedPointX) + 4*b)/2;
    printf("alpha1 %le\n alpha2 %le\n",alpha1,alpha2);
    /*the fisrt component of each eta is the second component of fixedPoint*/
    eta1[0] = 0.3*fixedPointX;
    eta2[0] = 0.3*fixedPointX;
    
    /* checking of alpha are right o not where alpha²+2*eta0*alpha/b -b is the equation for eigenvalues.
    printf("alpha2-- %le\n",alpha2*alpha2+2*a*eta2[0]*alpha2/b-b);
    printf("alpha1-- %le\n",alpha1*alpha1+2*a*eta1[0]*alpha1/b-b);
    */
    
    //now, each of etas contains the coefficient of the expansion of each corresponding Manifolds
    componentsManifold(eta2, alpha1, alpha2);
    componentsManifold(eta1, alpha2, alpha1);
    
    //we are going to write separaly the each component of estable and inestable manifold in diferent files. 
    for(i = 0; i < N; i++){
        fprintf(file1,"%d %le\n",i,eta1[i]);
        fprintf(file2,"%d %le\n",i,eta2[i]);
    }
    
    // Using fundamDomain we obtain two delta1s and we choose the minimum between
    delta1f = fundamDomain(eta1,alpha1,1);
    printf("delta1f %le\n",delta1f);
    delta1b = fundamDomain(eta1,alpha1,-1);
    printf("delta1b %le\n",delta1f);
    if (fabs(delta1b) < delta1f){
        delta1 = fabs(delta1b);
    }else{
        delta1 = delta1f;
    }
    
    //Same way to alpha2
    printf("delta1 %le \n",delta1);
    delta2f = fundamDomain(eta2,alpha2,1);
    printf("delta2f %le\n",delta2f);
    delta2b = fundamDomain(eta2,alpha2,-1);
    printf("delta2b %le\n",delta2b);
    if (fabs(delta2b) < delta2f){
        delta2 = fabs(delta2b);
    }else{
        delta2 = delta2f;
    }
    
    /*since we have the fundamental domain [-delta1, delta1] and [delta2, delta2] for ManiInestable and ManiEstable*/
    //checking 
    /*for (s= 0; s< delta1*pow(fabs(alpha1),5); s+= 0.001){
        evaluate(z,s,eta1,alpha1,delta1);
        fprintf(file4,"%le %le %le \n",s, z[0], z[1], functionalEqErr(eta1,alpha1,s));
    }
    for (s= 0; fabs(s)< delta1*pow(fabs(alpha1),5); s-= 0.001){
        evaluate(z,s,eta1,alpha1,delta1);
        fprintf(file4,"%le %le %le %le \n",s, z[0], z[1], functionalEqErr(eta1, alpha1, s));
    }*/
    
    /*Henon's Attractor*/
    //first we do a transient such as 10000 steps
    z0[0] = fixedPointX + apprX;
    z0[1] = 0.3*fixedPointX + apprY;
    Fn(z0, z1, 10000);
    z0[0] = z1[0];
    z0[1] = z1[1];
    
    for(i = 0; i < 10000; i++){
       Fn(z0, z1, 1);
       fprintf(file3,"%le %le\n", z1[0],z1[1]); 
       z0[0] = z1[0];
       z0[1] = z1[1];
    }
    
    //print the points of Manfolds
    Manifold(eta1, 0, alpha1, delta1, 1, -1);
    Manifold(eta1, 0, alpha1, delta1, -1, -1);
    Manifold(eta2, 0, alpha2, delta2, 1, 1);
    Manifold(eta2, 0, alpha2, delta2, -1, 1);
    
    
    fclose(file1);
    fclose(file2);
    fclose(file3);
    free(eta1);
    free(eta2);
    return 0;
}

int componentsManifold(double *C2,double alpha12, double alpha21){
    /*This function return the coffiecient of the expansion of the manifolds using a explicit formulate to calculate these.
     C is where we are going to save the coefficient and  
        The first component of expansion is the the sencond compnent of the fixed point that que chose.
        The second compnent of expansion is arbitrary, we choose 1
        
     alpha12 and alpha21 are eigenvalues of fixed point that we chose and there are 2 cases:
        case1:
            alpha12 = alpha1 and alpha21 = alpha2  this function returns the coefficient for Estable Manifold.
        case2:
            alpha12 = alpha2 and alpha12 = alpha1 this function returns the coefficient for Inestable Manifold. 
     */
    int n,j,k;
    double beta, an;
    
    C2[1] = 1;
    an= alpha21;
    for(n = 2; n < N; n++){
        an*= alpha21;
        beta = -a/(b*an*(1-alpha12/an)*(1-alpha21/an));
        C2[n] = 0;
        for(k = 1; k < n; k++){
            C2[n] +=C2[k]*C2[n-k];
        }
        C2[n] = C2[n]*beta;
    }
    return 0;
    
}

double fundamDomain(double *C,double alpha, int indicator){
    /*this function return a fundamental domain where the error of functional equation is small that an  epsilon that is given
        
        C is the expansion of Manifold could be the estable one or the inestable.
        alpha corresponds to the eigenvalue of estable or inestable Manifold.
        indicator could be 1 or -1
            case indicator = 1
                return [0,delta]
            case indicator = -1
                return [-delta ,0]
     */
    int  count;
    double I1, I2, pn, delta, err;
    
    delta = 0;
    count = 0;
    I1 = 0;
    I2 = indicator*1;
    pn = (I1 + I2) / 2 ;
    
    
    do{
        if((err= functionalEqErr(C,alpha,pn)) < error){
            delta = pn;
            I1 = pn;
            pn = (I1 + I2)/2;
            /*if(indicator ){
            printf("count %d pn %le err %le\n",count,pn, err);
            }*/
        }else{
            I2 = pn;
            pn = (I1 + I2) /2;
            /*if(indicator ){
            printf("count %d pn %le err %le\n",count,pn, err);
            }*/
        }
        
        count++;
    }while(count<10);
    return delta;
}

double functionalEqErr(double *C, double alpha, double t){
    /*This funciton returns the error of the functional equation at point t
      the parameters C, alpha are the same as before
     */
    double sol, aux1, aux2, aux3;
    int i;
    
    aux1 = 0;
    aux2 = 0;
    aux3 = 0;
  
    aux1 = hornerEvaluation(C,t);
    aux2 = hornerEvaluation(C,alpha*t);
    aux3 = hornerEvaluation(C,alpha*alpha*t);

    sol = fabs(aux1 +1 - a*aux2*aux2/(b*b)-aux3/b);
    return sol;
    
}

void Manifold(double *C, double s0, double alpha, double delta, int indicator, int TypeMani){
    /*This function returns the estable Manifold(TypeMani = 1)  or Inestable Manifold(TypeMani = -1) depen on the parameter TyperMani 
     given C,alpha,delta as before
     where indicator the points obtained are foward or backward
     */
    int i;
    double  z0[2], z1[2], d, dmin, dmax, longitud, dif;
    
    dmin = 1.e-4;
    dmax = 1.e-2;
    longitud = 0;
    dif = 0.001;
    FILE *file;
    
    
    
    if (TypeMani == 1){
        file = fopen("ManiEstable","w");
    }else{
        file = fopen("ManiInestable","w");
    }
    
    if(file == NULL  ){
        printf("problem in opening output files\n");
    }
    
    do{
        evaluate(z0, s0, C, alpha, delta); //z0 = ws0
        evaluate(z1, s0 + indicator*dif, C, alpha, delta); //z1 = ws1  
        d = hypot(z0[0]-z1[0], z0[1]-z1[1]); //hypotenuse = distance between z0 and z1
        
        if (d > dmax){
            dif /=  2;
        }else{
            fprintf(file,"%le %le %le \n",s0, z1[0], z1[1]);
            s0 += indicator*dif;
            longitud += d;
            if(d < dmin){
                dif *= 2;   
            }
        }
    }while(longitud < 1000);
    //we stop the do while  when th length of the curve is great than 1000
    fclose(file);
    return ;
}


void Fn(double z[2] ,double zn[2], int n){
    /*this funtion returns n-th evaluation of Henon Map at z=(z[0],z[1]) in a vector call zn = (zn[0], zn[1])
     Wath out! 
     if n > 0 we evaluate in F and if n < 0 we evaluate in inverse of F and n=0 we don't evaluate retu the same*/
    double x,y;
    int i;
    
    x= z[0];
    y = z[1];
    if (n > 0){
        for(i = 1; i <= n; i++){
            zn[0] = y +1 -a*x*x;
            zn[1] = b*x;
            x= zn[0];
            y = zn[1];
        }
    }else if(n == 0){
        zn[0] = x;
        zn[1] = y;
        
    }else{
        for(i = -1; i >= n; i--){
            zn[0] = y/b;
            zn[1] = x-1+a*y*y/(b*b);
            x = zn[0];
            y = zn[1];
        }
        
    }
    
    return;
}

void evaluate(double zn[2],double s, double *C, double alpha, double delta){
    /*This funciton returns w(s) where we work out and obtain w(s) = Fn(w(s/lambda^n))
     watch out !!
     if fabs(alpha) < 1 we need to multiply instead of divide*/
    int i;
    double aux,sn,n,x,y,z[2];
    if (fabs(s) <= delta){
        n = 0;
    }else{
        if (fabs(alpha) > 1){
            //this is the formulate obtained after working out the condition such that fabs(s/lambda^n) < delta
            n = ceil( log(fabs(s)/delta) / log(fabs(alpha)));
        }else{
            n = floor( log(fabs(s)/delta) / log(fabs(alpha)));
        }
    }
    s = s / pow(alpha, n);
    // evaluation in component x
    y = hornerEvaluation(C,s);
    //evaluation in component Y
    x = hornerEvaluation(C,alpha*s);
    x = x/b;
    z[0] = x;
    z[1] = y;
    
    Fn(z,zn,n);
}

double hornerEvaluation(double *C,double s0){
    double bn,bn_1;
    int i;
    
    bn = C[N];
    
    for(i = N; i > 0; i--){
        
      bn_1 = C[i-1]  + bn*s0;
      bn = bn_1;
        
    }
    return bn;
}