#include<math.h>
#include<stdio.h>
#include<stdlib.h>

#define error 1.e-10
#define tolerancia 1.e-16
#define N 25
#define pi acos(-1.0)
double a = 10.0;
double b = 28.0;
double c = 8.0/3.0;

void componentSolution(double *x, double *y, double *z);
double hornerEvaluation(double *C,double s0);
void integration(double *x, double *y,double *z,int indicator);

int main(void){
    /*LORENZ SYSTEM
        dx/dt = a(y-x)
        dy/dt = bx -xz -y
        dz/dt = xy-cz
    Main idea: Using the calculation that we have done, we are going to compute and sketch the Lorenz attractor.
    componentSolution: return the solution for initial condition (x0,y0,z0) 
    hornerEvaluation: evaluate a point in a polynomial.
    integration: using the method explained in memory. Given a initial condition s0 we calculate a approximation the solution of Lorenz system in s1 = s0 +h.
     */
    double x[N+3], y[N+3], z[N+3],alpha, alpha1; 
    
    FILE *file1;
    file1 = fopen("componentSolution","w");
    if(file1 == NULL){
        printf("there a problems in opening file\n");
        exit(-1);
    }
    //eigenvalue
    alpha = -((a+1)/2) + (sqrt((a+1)*(a+1) + 4*a*(b-1)))/2;
    alpha1 = -((a+1)/2) - (sqrt((a+1)*(a+1) + 4*a*(b-1)))/2;
    //eigenvector
    double v1[3] = {1, (alpha +a)/a, 0};
    double v2[3] = {1, (alpha1 +a)/a, 0};
    
   // printf("eigenvector =  % le % le %le \n eigenvalues = % le ",v[0],v[1],v[2],alpha);
    printf("enter the initial condition for the system: ");
    scanf("%lf %lf %lf", x, y, z);
    printf("initial conditions are %le %le %le\n",x[0],y[0],z[0]);    
    integration(x,y,z,2); //Given the initial condition by using integration we will able to get the point in attractor
    
    //Given a point that is a eigenvector1 multiply by 0.1, we using the intragration draw the orbital in positive time.
    x[0] = 0.1*v1[0];
    y[0] = 0.1*v1[1];
    z[0] = 0.1*v1[2];
    integration(x,y,z,1);
    
    //Given a point that is a eigenvector2 multiply by -0.1, we using the intragration draw the orbital in positive time.
    x[0] = -0.1*v2[0];
    y[0] = -0.1*v2[1];
    z[0] = -0.1*v2[2];
    integration(x,y,z,-1);
   
    return 0;

}

double hornerEvaluation(double *C, double x){
    double bn;
    int i;
    
    for(i= N, bn= C[N]; i > 0; i--)
      bn = C[i-1] + bn*x;
    
    return bn;
}
void integration(double *x, double *y,double *z,int indicator){
    //interation will be able to calculate Lorenz attractor(when indicator = 2) and Unstable manifold(indicator = -1 for eigenvector2 or indicator = -1 eigenvector2).
    double x0, y0, z0, dx, dy, dz,n;
    double h1, h2, h, t, tmax, dmax =0.1,dim = 0.0001;
    int bool;
    FILE *file1;

    if (indicator == 2){
        file1 = fopen("componentSolution","w");
	tmax= 200;
    }else if(indicator == 1){
        file1 = fopen("UnstableManifold1","w");
	tmax= 100;
    }else {
        file1 = fopen("UnstableManifold2","w");
	tmax= 100;
    }
    if(file1 == NULL){
      printf("there a problems in opening file\n");
      exit(-1);
    }

   
    for (t= 0; fabs(t)< tmax; ){
      
      componentSolution(x, y, z);
      // where h1 satnds for the error with coefficient of the n+1-th term that we commit in each step and h2 with n+2.
      h1 = pow(tolerancia/sqrt(x[N+1]*x[N+1] + y[N+1]*y[N+1] + z[N+1]*z[N+1]), 1./(N+1));
      h2 = pow(tolerancia/sqrt(x[N+2]*x[N+2] + y[N+2]*y[N+2] + z[N+2]*z[N+2]), 1./(N+2));
      //take the minimum
      h= (h1 < h2) ? h1 : h2;


      //(x0,y0,z0) stands the flow at the point (x0,y0,z0) after occuring t time.
      x0= x[0];
      y0= y[0];
      z0= z[0];
      
      //We vanish the the first component in each vector x,y,z in order to work out the better h.
      x[0]= 0;
      y[0]= 0;
      z[0]= 0;
      
      //if the the norm of the system is great that dmax we will reduce h.
      while( dx= hornerEvaluation(x, h),
	     dy= hornerEvaluation(y, h),
	     dz= hornerEvaluation(z, h),
	     sqrt(dx*dx+dy*dy+dz+dz)>dmax ){
	h/= 2.;
      }
      while( dx= hornerEvaluation(x, h),
	     dy= hornerEvaluation(y, h),
	     dz= hornerEvaluation(z, h),
	     sqrt(dx*dx+dy*dy+dz+dz) <dim){
	h*= 2.;
      }

      //dx is obtained evaluating in h in x with out the first component, respect to dy,dz are obtained in same way.Now, x[0],y[0],z[0] are got by evaluating at h and initial in flow condition(x0,y0,z0).In a addition it will be the new initial condition for the next round.
      
        x[0]= x0 + dx;
        y[0]= y0 + dy;
        z[0]= z0 + dz;
        t += h;

	if (!(indicator== 2 & fabs(t)<100)) {
	  fprintf(file1, "% le % le % le % le\n", t, x[0], y[0], z[0]);
	}
    }   
    fclose(file1);
    return ;
}

void componentSolution(double *x, double *y, double *z){
    //using the formula we worked out in memory
    int k, j;
    double xz,xy;
    
    for(k = 1; k <= N+2; k++){
         xz = 0;
         xy = 0;
        for(j = 0; j < k; j++){
            xz += x[j]*z[k-1-j];
            xy += x[j]*y[k-1-j];
        }
        x[k] = (a*(y[k-1] -x[k-1]))/k;
        y[k] = (b*x[k-1] -y[k-1] -xz)/k;
        z[k] = (xy -c*z[k-1])/k;

    }
}

