#include<math.h>
#include<stdio.h>
#include<stdlib.h>

#define error 1.e-10
#define tolerancia 1.e-16
#define NW 64
#define pi acos(-1.0)
double a = 10.0;
double b = 28.0;
double c = 8.0/3.0;

double Horner1D(double *C, double x, int N);
double Horner2D(double **W, double u, double v, int N);
double errFunctionalEq(double **Wx,double **Wy,double **Wz,double *alpha,double u,double v,int N);
double Horner2D_1(double **W, double u, double v, double *alpha,int N);

int main(void){
    /*LORENZ SYSTEM
        dx/dt = a(y-x)
        dy/dt = bx -xz -y
        dz/dt = xy-cz
    Main idea: to sketch the twe-dimensional stable manifold
    Horner2D: evaluate u,v in a homegeneous polynomial up to term N.
     */
    double **Wx, **Wy, **Wz;
    double alpha[3], M[3][3]= { {-a,a,0},{b,-1,0},{0,0,-c} }, P[3][3];
    int i, j, k, l, i1, i2, j1, j2, k0;
    double xz, xy, A, B, C, D, E, F, norm, err;
    double wx,wy, wz,du,dv,ut,vt;
    double check, u, v, r,t, th, maxerr;
   
    FILE *file1;
    FILE *file2;
    FILE *file3;
    file1 = fopen("Manifold","w");
    file2 = fopen("reduceDynamic","w");
    file3 = fopen("Dynamic","w");
    if(file1 == NULL || file2 == NULL){
        printf("there a problems in opening file\n ");
        exit (-1);
    } 

/*we save the component x,y,z of the satble manifold in W, Wy, Wz respectively.
  where Wx[k] is the vector for saving the coeffiient of the homogeneous polynomial of degree k and Wx[k][l] the coefficient of the term u^l v^(k-l) of the homogeneous polinomial of degree k.Same to Wy,Wz.  
  */
    Wx= (double **) malloc((NW+3)*sizeof(double*));
    Wy= (double **) malloc((NW+3)*sizeof(double*));
    Wz= (double **) malloc((NW+3)*sizeof(double*));

    for(k = 0; k<= NW+2; k++){
        Wx[k]= (double*) malloc((k+1)*sizeof(double));
        Wy[k]= (double*) malloc((k+1)*sizeof(double));
        Wz[k]= (double*) malloc((k+1)*sizeof(double));

        if (!Wx[k] || !Wy[k] || !Wz[k]) {
        puts("malloc problems");
        exit(-1);
        }
    }


    
    alpha[0] = -((a+1)/2) - (sqrt((a+1)*(a+1) + 4*a*(b-1)))/2;
    //printf("alpha 0: %lf\n", alpha[0]);
    P[0][0] = 1;
    P[1][0]= (alpha[0]-M[0][0])/M[0][1];
    P[2][0] = 0;
    
    alpha[1] = -c;
    //printf("alpha 1: %lf\n", alpha[1]);
    P[0][1]= 0;
    P[1][1]= 0;
    P[2][1]= 1;
        
    alpha[2] = -((a+1)/2) + (sqrt((a+1)*(a+1) + 4*a*(b-1)))/2;
    P[0][2] = 1;
    P[1][2]= (alpha[2]-M[0][0])/M[0][1];
    P[2][2] = 0;

    
    //normalize the eigenvector 
    for (j= 0; j<3; j++){
        norm= 0;
        for (i= 0; i<3; i++) {
        norm+= P[i][j]*P[i][j];
        }
        norm= sqrt(norm);
        for (i= 0; i<3; i++) {
        P[i][j]/= norm;
        }
    }
    
    // order 0 : the fix point
    Wx[0][0]= 0.;
    Wy[0][0]= 0.;
    Wz[0][0]= 0.;

    // order 1: the first column of P is the eigenvector for alpha1, the second for alpha2
    Wx[1][0]= P[0][1]; Wx[1][1]= P[0][0];
    Wy[1][0]= P[1][1]; Wy[1][1]= P[1][0];
    Wz[1][0]= P[2][1]; Wz[1][1]= P[2][0];
    
    for (k= 2; k<= NW+2; k++){
        for (l= 0; l<= k; l++) {
        // Equation for W[k][l]
        
        // Computation of xy, xz 
        xy= xz= 0;
        for (i1= 0; i1<= k; i1++) {
            for (j1= 0; j1<= l; j1++) {
            i2= k-i1;
            j2= l-j1;

            if (j1<= i1 && j2<= i2) {
                xy+= Wx[i1][j1]*Wy[i2][j2];
                xz+= Wx[i1][j1]*Wz[i2][j2];
            }
            }
        }
      
    Wz[k][l]= xy/(c+alpha[0]*l+alpha[1]*(k-l));
    //Using the Cramer rule to solve the linaer system
    A= -a-alpha[0]*l-alpha[1]*(k-l); B= a;
    C= b; D= -1-alpha[0]*l-alpha[1]*(k-l);
    E= 0; F= xz;

    Wx[k][l]= -F*B/(A*D-B*C);
    Wy[k][l]=  F*A/(A*D-B*C);

        }
    }

    //compute the 2D stable manifold, given points from balls center at the origin with radius r 1<= r <=20
    
    for (r= 1; r<= 20; r+= 1){
        maxerr= 0;
        for (th= 0; th<=2*M_PI+0.05; th+= 0.1) {
            u = r*cos(th);
            v = r*sin(th);      
            wx = Horner2D(Wx,u,v,NW+2);
            wy = Horner2D(Wy,u,v,NW+2);
            wz = Horner2D(Wz,u,v,NW+2);

            fprintf(file1,"% le % le % le % le % le % le\n", u, v,
                    Horner2D(Wx,u,v,NW+2),Horner2D(Wy,u,v,NW+2),Horner2D(Wz,u,v,NW+2),
                err= errFunctionalEq(Wx,Wy,Wz,alpha,u,v,NW+2));  
            if (err>maxerr) maxerr= err; //maxerr stands for the maximum error in each ball 
        }
        fprintf(file1,"\n#% le % le\n", r, maxerr); 
    }
    
    //Reduce Dynamic, we give the points in the closure of the ball with radius 20
    for(th = 0; th <=2*M_PI+0.05; th+= 0.4){
        u = 20*cos(th);
        v = 20*sin(th);
        t = 0;
        fprintf(file2,"\n#% le \n",th);
        do{
            ut = exp(alpha[0]*t)*u;
            vt = exp(alpha[1]*t)*v;
            t += 0.001;
            //reduce dynamic 2D 
            fprintf(file2,"% le % le %le\n",t,ut,vt); 
            //the dynamic of the system
            fprintf(file3,"% le % le % le % le % le % le\n", u, v,
                Horner2D(Wx,ut,vt,NW+2),Horner2D(Wy,ut,vt,NW+2),Horner2D(Wz,ut,vt,NW+2),
                 errFunctionalEq(Wx,Wy,Wz,alpha,ut,vt,NW+2)); 
        }while(fabs(ut)> 0.01  || fabs(vt)> 0.01);
        
    }
  return 0;
  fclose(file1);
  fclose(file2);
  fclose(file3);

}

double Horner1D(double *C, double x, int N){
    double bn;
    int i;
    
    for(i= N, bn= C[N]; i > 0; i--)
      bn = C[i-1] + bn*x;
    
    return bn;
}


double Horner2D_1(double **W, double u, double v, double *alpha,int N)
{
    // Horner2D_1 returns DW(s)·R(s) where we reformulate(see memory) and the programa will return the truncated series evaluting in DW(s)·R(s) by using a kind of horner method
    double w= 0, C;
    int l1, l2;

    w= alpha[1]*N*W[N][0];
    
    for (l2= N; l2>0; l2--) {
        // C[l2-1]
        C= (alpha[0]*(N-l2+1)+ (l2-1)*alpha[1])*W[N][N-l2+1];
        for (l1= N-l2+1; l1>0; l1--){
        C= (alpha[0]*(l1-1)+ alpha[1]*(l2-1))*W[l2+l1-2][l1-1] + C*u;
        }

        w= C + w*v;
    }
        
  return w;
}


double Horner2D(double **W, double u, double v, int N)
{
    //Horner2D returns F(K(s)) by using a kind of horner method in 2 variable 
    double w= 0, C;
    int l1, l2;

    w= W[N][0];
    
    for (l2= N; l2>0; l2--) {
        // C[l2-1]
        C= W[N][N-l2+1];
        for (l1= N-l2+1; l1>0; l1--){
        C= W[l2+l1-2][l1-1] + C*u;
        }

        w= C + w*v;
    }
        
    return w;
}

double errFunctionalEq(double **Wx,double **Wy,double **Wz,double *alpha,double u,double v,int N){
    //this program will return the modul of error of functional equation by using Horner2D and Horner2D_1.
    double wx,wy,wz,dwx,dwy,dwz,errx,erry,errz,errFunctionalEQ;

    wx = Horner2D(Wx,u,v,N);
    wy = Horner2D(Wy,u,v,N);
    wz = Horner2D(Wz,u,v,N);

    dwx = Horner2D_1(Wx,u,v,alpha,N);
    dwy = Horner2D_1(Wy,u,v,alpha,N);
    dwz = Horner2D_1(Wz,u,v,alpha,N);

    errx = pow(fabs(a*(wy-wx) - dwx),2);
    erry = pow(fabs(wx*(b-wz)-wy - dwy),2);
    errz = pow(fabs(wx*wy-c*wz - dwz),2);
    errFunctionalEQ = sqrt(errx + erry + errz);
    
    return errFunctionalEQ;   
}
